    .586
    .model flat, stdcall
    option casemap :none

    include \masm32\include\kernel32.inc
    include \masm32\include\windows.inc

; 
;
; BUFFER HANDLING
;
; -------------------------------------------------------------------------

    include buffer.inc


.code
; -------------------------------------------------------------------------
AllocBuffer     PROC USES esi lpBuffer:DWORD,dwSize:DWORD
                ASSUME esi:PTR BUFFER

                invoke GlobalAlloc,GHND,dwSize
                or      eax,eax
                jz  @_AllocBuffer_End
; ----------------
                mov     esi,lpBuffer
                mov     [esi].HND_,eax
                xor     eax,eax
                mov     [esi].START_,eax
                mov     [esi].CURR_,eax
                mov     [esi].END_,eax
                mov     eax,dwSize
                mov     [esi].SIZE_,eax

@_AllocBuffer_End:
                ret

                ASSUME esi:NOTHING
AllocBuffer     ENDP
; -------------------------------------------------------------------------
FreeBuffer      PROC USES esi lpBuffer:DWORD
                ASSUME esi:PTR BUFFER

                mov     esi,lpBuffer
                mov     eax,[esi].HND_
                or      eax,eax
                jz  @_FreeBuffer_End                
                invoke GlobalFree,eax

@_FreeBuffer_End:
                ret

                ASSUME esi:NOTHING
FreeBuffer      ENDP

; 
;
; Lock buffer - locks a moveable memory block if not locked before
;               and returns the start address
;
;  IN:      lpBuffer - address of BUFFER structure
; OUT:      eax - starting address of buffer
; -------------------------------------------------------------------------

LockBuffer      PROC USES esi lpBuffer:DWORD
                ASSUME esi:PTR BUFFER

                mov     esi,lpBuffer
                mov     eax,[esi].START_
                or      eax,eax
                jnz @_LockBuffer_End
                mov     eax,[esi].HND_
                or      eax,eax
                jz  @_LockBuffer_End
                invoke GlobalLock,eax
                mov     [esi].START_,eax
                mov     [esi].CURR_,eax
                add     eax,[esi].SIZE_
                mov     [esi].END_,eax

@_LockBuffer_End:
                ret

                ASSUME esi:NOTHING
LockBuffer      ENDP
; 
;
; Unlock buffer - unlocks a moveable memory block if not unlocked before
;
;  IN:      lpBuffer - address of BUFFER structure
; OUT:      eax - NULL if unlocked
;                 
; -------------------------------------------------------------------------

UnlockBuffer    PROC USES esi lpBuffer:DWORD
                ASSUME esi:PTR BUFFER

                mov     esi,lpBuffer
                mov     eax,[esi].START_
                or      eax,eax
                jz  @_UnlockBuffer_End
                mov     eax,[esi].HND_
                or      eax,eax
                jz  @_UnlockBuffer_End
                invoke GlobalUnlock,eax
                or      eax,eax
                jnz @_UnlockBuffer_End
                invoke GetLastError
                cmp     eax,NO_ERROR
                jnz @_UnlockBuffer_End
                xor     eax,eax
                mov     [esi].HND_,eax
                mov     [esi].START_,eax
                mov     [esi].CURR_,eax
                mov     [esi].END_,eax

@_UnlockBuffer_End:
                ret

                ASSUME esi:NOTHING
UnlockBuffer    ENDP

end