; #########################################################################

;           Assembler specific instructions for 32 bit ASM code

      .386                   ; minimum processor needed for 32 bit
      .model flat, stdcall   ; FLAT memory model & STDCALL calling
      option casemap :none   ; set code to case sensitive

; #########################################################################

      include \masm32\include\windows.inc
      include resource\resource.inc

      ; -------------------------------------------------------------

      include \masm32\include\user32.inc
      include \masm32\include\kernel32.inc
      include \masm32\include\comdlg32.inc
      include \masm32\macros\macros.asm

      include \masm32\include\winmm.inc
      include dsound\dx9macros.inc
      include dsound\dsound9.inc
  
        include \masm32\include\masm32.inc
        include \masm32\include\debug.inc

      include misc\buffer.inc
  
      include sound.inc


      includelib \masm32\lib\user32.lib
      includelib \masm32\lib\kernel32.lib
      includelib \masm32\lib\comdlg32.lib

      includelib misc\buffer.lib
      includelib dsound\dsound.lib

        includelib \masm32\lib\debug.lib
        includelib \masm32\lib\masm32.lib



; #########################################################################

      ; 1. szText

      szText MACRO Name, Text:VARARG
        LOCAL lbl
          jmp lbl
            Name db Text,0
          lbl:
        ENDM

      ; 2. m2m

      m2m MACRO M1, M2
        push M2
        pop  M1
      ENDM

      ; 3. return

      return MACRO arg
        mov eax, arg
        ret
      ENDM

; #########################################################################

        WinMain PROTO :DWORD,:DWORD,:DWORD,:DWORD
        WndProc PROTO :DWORD,:DWORD,:DWORD,:DWORD
        TopXY PROTO   :DWORD,:DWORD

; #########################################################################


 .data
 
    szDisplayName   db "Sound Play",0
    CommandLine     dd 0
    hWnd            dd 0
    hInstance       dd 0

    ofn   OPENFILENAME <>
    OpenBoxTitle    db "Open...",0
    fileFilter      db "raw samples",0,"*.pcm",0
                    db "all files",0,"*.*",0,0


 inputFile LABEL BYTE
    fileHandle          dd ?
    fileSize            dd ?
    fileName            db 260 dup (0)

; errorBox
    errorText1          db 'AllocBuffer failed!',0
    errorText2          db 'LockBuffer failed!',0
    errorText3          db 'UnlockBuffer failed!',0
    errorText4          db 'FreeBuffer failed!',0
    errorText5          db 'File not found!',0
    errorText6          db 'File read error!',0
    errorText7          db 'File not created!',0
    errorText8          db 'File write error!',0
    errorText9          db 'File close error!',0
    errorText10         db 'Thread not created!',0
    errorBox            db 'Message...',0

; #########################################################################


max_buffer_size EQU 0100000h            ; max size is 1MB


    .code


start:
    invoke GetModuleHandle, NULL ; provides the instance handle
    mov hInstance, eax

    invoke GetCommandLine        ; provides the command line address
    mov CommandLine, eax

    invoke WinMain,hInstance,NULL,CommandLine,SW_SHOWDEFAULT
    
    invoke ExitProcess,eax       ; cleanup & return to operating system

; #########################################################################


; 

FetchFile   PROC USES ebx ecx esi lpFileData:DWORD,lpBuffer:DWORD

; Read file
            mov     esi,lpFileData
            lea     ebx,[esi+8]
            invoke CreateFile,ebx,GENERIC_READ,0,0,OPEN_EXISTING,0,NULL
            cmp     eax,INVALID_HANDLE_VALUE
            jnz @_Load_file
                invoke MessageBox,NULL,addr errorText5,addr errorBox,MB_OK
                xor     eax,eax
                jmp @_GetFile_END
; ----------------
@_Load_file:
            mov     [esi],eax
            lea     eax,[esi+4]
            invoke GetFileSize,[esi],eax
            cmp     eax,max_buffer_size
            jc  @_fileSizeOk
            mov     eax,max_buffer_size
@_fileSizeOk:
            mov     ebx,lpBuffer
            lea     ecx,[esi+4]
            ASSUME ebx:PTR BUFFER
            invoke ReadFile,[esi],[ebx].START_,eax,ecx,NULL
            or      eax,eax
            jnz @_Close_file
                invoke MessageBox,NULL,addr errorText6,addr errorBox,MB_OK
                xor     eax,eax
                jmp @_GetFile_END
; ----------------
@_Close_file:
            invoke CloseHandle,[esi]
            or      eax,eax
            jnz @_GetFile_END
                invoke MessageBox,NULL,addr errorText9,addr errorBox,MB_OK
                xor     eax,eax
                jmp @_GetFile_END
; ----------------
@_GetFile_END:
            ret

FetchFile   ENDP


; 

InitAll     PROC

            invoke AllocBuffer,addr SampleBuffer,max_buffer_size
            or      eax,eax
            jnz  @_BufferAllocated
                invoke MessageBox,NULL,addr errorText1,addr errorBox,MB_OK
                xor     eax,eax
                jmp @_InitAll_END
; ----------------
@_BufferAllocated:
            invoke LockBuffer,addr SampleBuffer
            or      eax,eax
            jnz  @_BufferLocked
                invoke MessageBox,NULL,addr errorText2,addr errorBox,MB_OK
                xor     eax,eax
                jmp @_InitAll_END
; ----------------
@_BufferLocked:

            invoke InitDSound,hWnd,44100,1,16
            mov     ebx,[eax]
            add     eax,4
            invoke MessageBox,NULL,eax,addr errorBox,MB_OK
            xor     eax,eax
            or      ebx,ebx
            jnz  @_InitAll_END

            invoke StartDSound
            mov     eax,1

@_InitAll_END:
            ret
InitAll     ENDP

; #########################################################################

WinMain proc hInst:DWORD,hPrevInst:DWORD,CmdLine:DWORD,CmdShow:DWORD

        ;====================
        ; Put LOCALs on stack
        ;====================

        LOCAL wc   :WNDCLASSEX
        LOCAL msg  :MSG

        LOCAL Wwd  :DWORD
        LOCAL Wht  :DWORD
        LOCAL Wtx  :DWORD
        LOCAL Wty  :DWORD


        szText szClassName,"Generic_Class"

        ;==================================================
        ; Fill WNDCLASSEX structure with required variables
        ;==================================================

        mov wc.cbSize,         sizeof WNDCLASSEX
        mov wc.style,          CS_HREDRAW or CS_VREDRAW \
                               or CS_BYTEALIGNWINDOW
        mov wc.lpfnWndProc,    offset WndProc      ; address of WndProc
        mov wc.cbClsExtra,     NULL
        mov wc.cbWndExtra,     NULL
        m2m wc.hInstance,      hInst               ; instance handle
        mov wc.hbrBackground,  COLOR_BTNFACE+1     ; system color
        mov wc.lpszMenuName,   NULL
        mov wc.lpszClassName,  offset szClassName  ; window class name
          invoke LoadIcon,hInst,500    ; icon ID   ; resource icon
        mov wc.hIcon,          eax
          invoke LoadCursor,NULL,IDC_ARROW         ; system cursor
        mov wc.hCursor,        eax
        mov wc.hIconSm,        0

        invoke RegisterClassEx, ADDR wc     ; register the window class

        ;================================
        ; Centre window at following size
        ;================================

        mov Wwd, 500
        mov Wht, 350

        invoke GetSystemMetrics,SM_CXSCREEN ; get screen width in pixels
        invoke TopXY,Wwd,eax
        mov Wtx, eax

        invoke GetSystemMetrics,SM_CYSCREEN ; get screen height in pixels
        invoke TopXY,Wht,eax
        mov Wty, eax

        ; ==================================
        ; Create the main application window
        ; ==================================
        invoke CreateWindowEx,WS_EX_OVERLAPPEDWINDOW,
                              ADDR szClassName,
                              ADDR szDisplayName,
                              WS_OVERLAPPEDWINDOW,
                              Wtx,Wty,Wwd,Wht,
                              NULL,NULL,
                              hInst,NULL

        mov   hWnd,eax  ; copy return value into handle DWORD

        invoke LoadMenu,hInst,600                 ; load resource menu
        invoke SetMenu,hWnd,eax                   ; set it to main window

        invoke ShowWindow,hWnd,SW_SHOWNORMAL      ; display the window
        invoke UpdateWindow,hWnd                  ; update the display

      ;===================================
      ; Loop until PostQuitMessage is sent
      ;===================================

    StartLoop:
      invoke GetMessage,ADDR msg,NULL,0,0         ; get each message
      cmp eax, 0                                  ; exit if GetMessage()
      je ExitLoop                                 ; returns zero
      invoke TranslateMessage, ADDR msg           ; translate it
      invoke DispatchMessage,  ADDR msg           ; send it to message proc
      jmp StartLoop
    ExitLoop:

      return msg.wParam

WinMain endp

; #########################################################################

WndProc proc hWin:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD

    .if uMsg == WM_COMMAND
    
    ;======== menu commands ========

        .if wParam == IDQUIT
            invoke SendMessage,hWin,WM_SYSCOMMAND,SC_CLOSE,NULL

        .elseif wParam == IDOPEN
                mov     ofn.lStructSize,    sizeof OPENFILENAME
                m2m     ofn.hWndOwner,      hWin
                m2m     ofn.hInstance,      hInstance
                m2m     ofn.lpstrFilter,    offset fileFilter
                m2m     ofn.lpstrFile,      offset fileName
                mov     ofn.nMaxFile,       260
                m2m     ofn.lpstrTitle,     offset OpenBoxTitle
                mov     ofn.Flags,          OFN_EXPLORER or OFN_FILEMUSTEXIST or OFN_LONGNAMES

                invoke GetOpenFileName,addr ofn
                .if (eax)
                    invoke FetchFile,addr inputFile,addr SampleBuffer
                    mov     eax,DWORD PTR [inputFile+4]
                    mov     [SampleBuffer].SIZE_,eax
                    add     eax,[SampleBuffer].START_
                    mov     [SampleBuffer].END_,eax
                    m2m     [SampleBuffer].CURR_,[SampleBuffer].START_
                    add     eax,DWORD PTR [inputFile+4]
                .endif

        .elseif wParam == IDABOUT
            szText TheMsg,"Streaming SoundBuffer test"
            invoke MessageBox,hWin,ADDR TheMsg,ADDR szDisplayName,MB_OK
        .endif

    ;====== end menu commands ======

    .elseif uMsg == WM_CREATE

            m2m     hWnd,hWin

            invoke InitAll
            or      eax,eax
            jz  @_Terminating

            ; invoke StartDSound


    .elseif uMsg == WM_CLOSE
    ; -------------------------------------------------------------------
    ; This is the place where various requirements are performed before
    ; the application exits to the operating system such as deleting
    ; resources and testing if files have been saved. You have the option
    ; of returning ZERO if you don't wish the application to close which
    ; exits the WndProc procedure without passing this message to the
    ; default window processing done by the operating system.
    ; -------------------------------------------------------------------
    ;    szText TheText,"Please Confirm Exit"
    ;    invoke MessageBox,hWin,ADDR TheText,ADDR szDisplayName,MB_YESNO
    ;      .if eax == IDNO
    ;        return 0
    ;      .endif
          jmp @_Terminating

    .elseif uMsg == WM_DESTROY

; ----------------
@_Terminating:
            cmp     hMixThread,0
            jz  @_WndQuit
            mov     isTerminating,1
            invoke SetEvent,hNotification3
            invoke WaitForSingleObject,hMixThread,INFINITE
            invoke CloseHandle,hMixThread
PrintText 'terminated'
; ----------------
@_WndQuit:
            invoke UnlockBuffer,addr SampleBuffer
            or      eax,eax
            jz  @_BufferUnlocked
                invoke MessageBox,NULL,addr errorText3,addr errorBox,MB_OK
; ----------------
@_BufferUnlocked:
            invoke FreeBuffer,addr SampleBuffer
            or      eax,eax
            jz  @_END
                invoke MessageBox,NULL,addr errorText4,addr errorBox,MB_OK
; ----------------
@_END:
        invoke PostQuitMessage,NULL
        return 0 
    .endif

    invoke DefWindowProc,hWin,uMsg,wParam,lParam
    ; --------------------------------------------------------------------
    ; Default window processing is done by the operating system for any
    ; message that is not processed by the application in the WndProc
    ; procedure. If the application requires other than default processing
    ; it executes the code when the message is trapped and returns ZERO
    ; to exit the WndProc procedure before the default window processing
    ; occurs with the call to DefWindowProc().
    ; --------------------------------------------------------------------

    ret

WndProc endp

; ########################################################################

TopXY proc wDim:DWORD, sDim:DWORD

    ; ----------------------------------------------------
    ; This procedure calculates the top X & Y co-ordinates
    ; for the CreateWindowEx call in the WinMain procedure
    ; ----------------------------------------------------

    shr sDim, 1      ; divide screen dimension by 2
    shr wDim, 1      ; divide window dimension by 2
    mov eax, wDim    ; copy window dimension into eax
    sub sDim, eax    ; sub half win dimension from half screen dimension

    return sDim

TopXY endp

; ########################################################################

end start